/* $Id: $
 *
 * RSSEvent.cpp
 *
 * Author: Danny Reinhold / Reinhold Software Services
 *
 * Permission is hereby granted, free of charge, to any person
 * obtaining a copy of this software and associated documentation
 * files (the "Software"), to deal in the Software without restriction,
 * including without limitation the rights to use, copy, modify, merge,
 * publish, distribute, sublicense, and/or sell copies of the Software,
 * and to permit persons to whom the Software is furnished to do so,
 * subject to the following conditions: 
 * The above copyright notice and this permission notice shall be included
 * in all copies or substantial portions of the Software. 
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
 * EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
 * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
 * IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM,
 * DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE,
 * ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
 * DEALINGS IN THE SOFTWARE. 
 */

#include "iup.h"
#include "RSSEvent.h"
#include "RSSWidgetManager.h"


RSSEvent::RSSEvent(std::string actionId, RSSWidget *widget,
    std::string text, int index, bool selectionFlag)
  : m_actionId(actionId), m_widget(widget), m_text(text), m_index(index), m_selectionFlag(selectionFlag)
{
}


RSSEvent::~RSSEvent()
{
}


std::string RSSEvent::getActionId()
{
  return m_actionId;
}


RSSWidget *RSSEvent::getWidget()
{
  return m_widget;
}


std::string RSSEvent::getText()
{
  return m_text;
}


int RSSEvent::getIndex()
{
  return m_index;
}


bool RSSEvent::getSelectionFlag()
{
  return m_selectionFlag;
}


RSSEventHandler::RSSEventHandler()
{
}


RSSEventHandler::~RSSEventHandler()
{
}


void RSSEventHandler::registerCallback(RSSWidget *widget, RSSCallback callback)
{
  if(!widget)
    return;

  std::string actionId = widget->getDefaultActionId();
  m_registeredCallbacks[actionId] = callback;

  RSSEventManager *eventManager = RSSEventManager::getInstance();
  eventManager->registerEventHandler(widget->getCallbackType(), actionId, this);
}


int RSSEventHandler::processEvent(RSSEvent &event)
{
  int result = RSSGUIDEFAULT;

  std::string actionId = event.getActionId();

  if(m_registeredCallbacks.find(actionId) != m_registeredCallbacks.end())
  {
    RSSCallback callback = m_registeredCallbacks[actionId];
    if(callback)
    {
      result = (*this.*callback)(event);
    }
  }

  return result;
}



extern "C" int rssGuiGlobalCallback(Ihandle *handle, ...)
{
  if(!handle)
    return RSSGUIDEFAULT;

  RSSEventManager *eventManager = RSSEventManager::getInstance();

  std::string actionId = IupGetActionName();
  std::string widgetId = IupGetName(handle);

  RSSWidgetManager *widgetManager = RSSWidgetManager::getInstance();
  RSSWidget *widget = widgetManager->getWidget(widgetId);
  if(!widget)
    return RSSGUIDEFAULT;

  RSSEvent event(actionId, widget);

  return eventManager->processEvent(event);
}


extern "C" int rssGuiListCallback(Ihandle *handle, char *text, int index, int selectionFlag)
{
  if(!handle)
    return RSSGUIDEFAULT;

  RSSEventManager *eventManager = RSSEventManager::getInstance();

  std::string actionId = IupGetActionName();
  std::string widgetId = IupGetName(handle);

  RSSWidgetManager *widgetManager = RSSWidgetManager::getInstance();
  RSSWidget *widget = widgetManager->getWidget(widgetId);
  if(!widget)
    return RSSGUIDEFAULT;

  RSSEvent event(actionId, widget, text, index, selectionFlag ? true : false);

  return eventManager->processEvent(event);
}


RSSEventManager *RSSEventManager::getInstance()
{
  static RSSEventManager *instance = 0;

  if(!instance)
    instance = new RSSEventManager();

  return instance;
}


RSSEventManager::RSSEventManager()
{
}


RSSEventManager::~RSSEventManager()
{
}


void RSSEventManager::registerEventHandler(RSSCallbackType callbackType,
    std::string actionId, RSSEventHandler *eventHandler)
{
  m_registeredEventHandlers[actionId] = eventHandler;
  if(callbackType == RSSGUI_GLOBAL_CALLBACK)
    IupSetFunction((char *)actionId.c_str(), rssGuiGlobalCallback);
  else if(callbackType == RSSGUI_LIST_CALLBACK)
    IupSetFunction((char *)actionId.c_str(), (Icallback)rssGuiListCallback);
}


int RSSEventManager::processEvent(RSSEvent &event)
{
  int result = RSSGUIDEFAULT;

  RSSEventHandler *eventHandler;

  if(m_registeredEventHandlers.find(event.getActionId()) != m_registeredEventHandlers.end())
  {
    eventHandler = m_registeredEventHandlers[event.getActionId()];
    result = eventHandler->processEvent(event);
  }

  return result;
}


int RSSEventManager::mainLoop()
{
  return rssGuiMainLoop();
}


int RSSEventManager::loopStep()
{
  return rssGuiLoopStep();
}


