/* $Id: $
 *
 * RSSWidget.cpp
 *
 * Author: Danny Reinhold / Reinhold Software Services
 *
 * Permission is hereby granted, free of charge, to any person
 * obtaining a copy of this software and associated documentation
 * files (the "Software"), to deal in the Software without restriction,
 * including without limitation the rights to use, copy, modify, merge,
 * publish, distribute, sublicense, and/or sell copies of the Software,
 * and to permit persons to whom the Software is furnished to do so,
 * subject to the following conditions: 
 * The above copyright notice and this permission notice shall be included
 * in all copies or substantial portions of the Software. 
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
 * EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
 * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
 * IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM,
 * DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE,
 * ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
 * DEALINGS IN THE SOFTWARE. 
 */

#include <stdio.h>
#include <stdarg.h>
#include "iup.h"
#include "iuptabs.h"
#include "RSSWidget.h"
#include "RSSGlobals.h"
#include "RSSWidgetManager.h"


RSSWidget::RSSWidget(std::string widgetType, std::string widgetId)
  : m_widgetType(widgetType), m_widgetId(widgetId), m_handle(0)
{
  if(m_widgetId == "")
    m_widgetId = rssGuiGetNewWidgetId();

  m_defaultActionId = rssGuiGetNewActionId();

  RSSWidgetManager *widgetManager = RSSWidgetManager::getInstance();
  widgetManager->addWidget(this);
}


RSSWidget::~RSSWidget()
{
  RSSWidgetManager *widgetManager = RSSWidgetManager::getInstance();
  widgetManager->removeWidget(this);

  // Don't check m_handle here - it may be invalid because of
  // an IupDestroy call in a parent constructor...
  Ihandle *handle = IupGetHandle((char *)m_widgetId.c_str());
  if(handle)
    IupDestroy(handle);
  m_handle = 0;

  std::vector<RSSWidget *>::iterator it;
  for(it = m_subWidgets.begin(); it != m_subWidgets.end(); it++)
  {
    if(*it)
      delete *it;
  }
}


void *RSSWidget::getHandle()
{
  return m_handle;
}


std::string RSSWidget::getWidgetType()
{
  return m_widgetType;
}


std::string RSSWidget::getWidgetId()
{
  return m_widgetId;
}


std::string RSSWidget::getDefaultActionId()
{
  return m_defaultActionId;
}


void RSSWidget::setWidgetId(std::string newWidgetId)
{
  if(m_widgetId == "")
    return;

  RSSWidgetManager *widgetManager = RSSWidgetManager::getInstance();
  widgetManager->removeWidget(this);

  m_widgetId = newWidgetId;

  widgetManager->addWidget(this);

  if(m_handle)
    IupSetHandle((char *)(m_widgetId.c_str()), (Ihandle *)m_handle);

}


void RSSWidget::setExpandFlag(bool expandable)
{
  setAttribute(IUP_EXPAND, expandable ? IUP_YES : IUP_NO);
}


void RSSWidget::setToolTip(std::string newToolTip)
{
  setAttribute(IUP_TIP, newToolTip.c_str());
}


void RSSWidget::setTabTitle(std::string newTabTitle)
{
  setAttribute(ICTL_TABTITLE, newTabTitle.c_str());
}

void RSSWidget::setFocus()
{
  if(m_handle)
    IupSetFocus((Ihandle *)m_handle);
}


std::string RSSWidget::getAttribute(std::string attr)
{
  if(!m_handle)
  {
    std::string value = "";
    for(std::list<RSSAttr>::const_iterator it = m_attributes.begin(); it != m_attributes.end(); it++)
    {
      if(it->attribute == attr)
      {
        value = it->value;
        break;
      }
    }
    return value;
  }

  char *a = IupGetAttribute((Ihandle *)m_handle, (char *)attr.c_str());
  return a ? a : "";
}


void RSSWidget::setAttribute(std::string attr, std::string value)
{
  if(m_handle)
    IupStoreAttribute((Ihandle *)m_handle, (char *)attr.c_str(), (char *)value.c_str());
  else
  {
    RSSAttr attribute;
    attribute.attribute = attr;
    attribute.value     = value;
    m_attributes.push_back(attribute);
  }
}


void RSSWidget::deleteAttribute(std::string attr)
{
  if(m_handle)
    IupStoreAttribute((Ihandle *)m_handle, (char *)attr.c_str(), 0);
  else
  {
    std::list<RSSAttr>::iterator it;
    bool found = false;
    for(it = m_attributes.begin(); it != m_attributes.end(); it++)
      found = (*it).attribute == attr;
    if(found)
      m_attributes.erase(it);
  }
}



void RSSWidget::show()
{
  create(); // create the widget and its sub widgets if this has not been done...

  if(m_handle)
    IupShow((Ihandle *)m_handle);
}


void RSSWidget::hide()
{
  if(m_handle)
    IupHide((Ihandle *)m_handle);
}


std::vector<RSSWidget *> &RSSWidget::getSubWidgets()
{
  return m_subWidgets;
}


void RSSWidget::addSubWidget(RSSWidget *widget)
{
  if(!widget)
    return;

  m_subWidgets.push_back(widget);

  Ihandle *subWidgetHandle = (Ihandle *)widget->getHandle();

  if(m_handle && subWidgetHandle)
  {
    IupAppend((Ihandle *)m_handle, subWidgetHandle);
    m_alreadyAddedSubWidgets.insert(widget->getWidgetId());
  }
}


void RSSWidget::addSubWidgets(RSSWidget *widget, ...)
{
  va_list args;
  va_start(args, widget);

  RSSWidget *currentWidget = widget;
  while(currentWidget)
  {
    addSubWidget(currentWidget);
    currentWidget = va_arg(args, RSSWidget *);
  }

  va_end(args);
}


void RSSWidget::detachSubWidget(RSSWidget *widget)
{
  bool found = false;
  std::vector<RSSWidget *>::iterator it;

  for(it = m_subWidgets.begin(); it != m_subWidgets.end() && !found; it++)
    found = *it == widget;

  if(found)
  {
    m_subWidgets.erase(it);

    Ihandle *subWidgetHandle = (Ihandle *)widget->getHandle();
    if(m_handle && subWidgetHandle)
    {
      IupDetach(subWidgetHandle);
      std::set<std::string>::iterator sit;
      sit = m_alreadyAddedSubWidgets.find(widget->getWidgetId());
      if(sit != m_alreadyAddedSubWidgets.end())
        m_alreadyAddedSubWidgets.erase(sit);
    }
  }
}

bool RSSWidget::create()
{
  bool success = true;

  if(m_handle)
    return true;

  // create sub widgets if neccessary
  std::vector<RSSWidget *>::iterator it;
  for(it = m_subWidgets.begin(); it != m_subWidgets.end() && success; it++)
    success = (*it)->create();

  if(!success)
    return false;

  success = createLowLevelWidget();
  if(!success || !m_handle)
    return false;
  IupSetHandle((char *)m_widgetId.c_str(), (Ihandle *)m_handle);

  // append sub widgets that are not already appended
  for(it = m_subWidgets.begin(); it != m_subWidgets.end(); it++)
  {
    RSSWidget *child = *it;
    std::string childId = child->getWidgetId();
    if(m_alreadyAddedSubWidgets.find(childId) == m_alreadyAddedSubWidgets.end())
    {
      IupAppend((Ihandle *)m_handle, (Ihandle *)child->getHandle());
      m_alreadyAddedSubWidgets.insert(childId);
    }
  }

  // set attributes
  std::list<RSSAttr>::iterator ait;
  for(ait = m_attributes.begin(); ait != m_attributes.end(); ait++)
  {
    RSSAttr &a = *ait;
    setAttribute(a.attribute, a.value);
  }
  m_attributes.clear();

  return success;
}


RSSCallbackType RSSWidget::getCallbackType()
{
  return RSSGUI_GLOBAL_CALLBACK;
}


