/* $Id: $
 *
 * RSSEvent.h
 *
 * The event manager and helper classes...
 *
 * Author: Danny Reinhold / Reinhold Software Services
 *
 * Permission is hereby granted, free of charge, to any person
 * obtaining a copy of this software and associated documentation
 * files (the "Software"), to deal in the Software without restriction,
 * including without limitation the rights to use, copy, modify, merge,
 * publish, distribute, sublicense, and/or sell copies of the Software,
 * and to permit persons to whom the Software is furnished to do so,
 * subject to the following conditions: 
 * The above copyright notice and this permission notice shall be included
 * in all copies or substantial portions of the Software. 
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
 * EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
 * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
 * IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM,
 * DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE,
 * ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
 * DEALINGS IN THE SOFTWARE. 
 */

#ifndef RSSEVENT_H
#define RSSEVENT_H

#pragma warning(disable:4786)

#include <map>
#include <string>
#include "RSSGuiDeclSpec.h"
#include "RSSWidget.h"
#include "RSSGlobals.h"

// Just a little macro that saves a bit of typing
// instead of writing
// registerCallback(b1, (RSSCallback)(&MyClass::button1Callback))
// you can simply write
// rssRegisterEvent(b1,MyClass::button1Callback)
// or vice versa
#define rssRegisterEvent(W,X) registerCallback(W, (RSSCallback)(&X))


// Every event is described by an instance of class RSSEvent.
// Even if several widget have specialized information (as
// RSSList knows which item is selected or deselected) we
// decided to not create subclasses for those special events,
// but handle them with this class.
class RSS_GUI_DECL_SPEC RSSEvent
{
  public:
    RSSEvent( std::string actionId, RSSWidget *widget,
        std::string text = "", int index = 0, bool selectionFlag = true);
    virtual ~RSSEvent();

    virtual RSSWidget   *getWidget();
    virtual std::string  getActionId();

    // only applicable for list events...
    virtual std::string  getText();          // the text of the affected item
    virtual int          getIndex();         // the index of the affected item
    virtual bool         getSelectionFlag(); // is the item now selected or deselected?

  private:
    std::string  m_actionId;
    RSSWidget   *m_widget;

    std::string  m_text;
    int          m_index;
    bool         m_selectionFlag;
};


class RSS_GUI_DECL_SPEC RSSEventHandler;
typedef int (RSSEventHandler::*RSSCallback)(RSSEvent &event);


class RSS_GUI_DECL_SPEC RSSEventHandler
{
  public:
    RSSEventHandler();
    virtual ~RSSEventHandler();

    void registerCallback(RSSWidget *widget, RSSCallback callback);

    virtual int processEvent(RSSEvent &event);

  protected:
    std::map<std::string, RSSCallback> m_registeredCallbacks;
};


class RSS_GUI_DECL_SPEC RSSEventManager
{
  public:
    static RSSEventManager *getInstance();

    // register an event handler
    // if the event 'actionId' occurs, the method processEvent()
    // of the event handler gets called...
    void registerEventHandler(RSSCallbackType callbackType, std::string actionId, RSSEventHandler *);

    // start the main loop of RSSGui - this method blocks
    // the calling thread
    int mainLoop();

    // process one or more events in the event queue
    // if no events have to be processed, the function returns
    // this way you can realize an own main loop - simply call
    // this function regularly...
    int loopStep();

    // for internal use only...
    int processEvent(RSSEvent &event);

  private:
    RSSEventManager();
    ~RSSEventManager();

    std::map<std::string, RSSEventHandler *> m_registeredEventHandlers;
};


#endif


